/*
    example3b.c: Verify a secret on a SmartDongle

    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704

    www.smartdongle.com


    ***** BEGIN LICENSE BLOCK *****

    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is MicroWorks, Inc. code.

    The Initial Developer of the Original Code is MicroWorks, Inc. Portions
    created by MicroWorks, Inc. are Copyright (C) 2008
    MicroWorks, Inc. All Rights Reserved.

    Contributor(s):

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK *****
*/
#include <stdio.h>
#include <string.h>
#include "smartdongle.h"
#include "aes.h"

int main ( int argc, char **argv) {

    /* SmartDongle demo keys. */
    unsigned long long P1 = 0xec6cc589aefd1e75ULL;
    unsigned long long P2 = 0xfcec0a6a82747b3fULL;

    /*
        This is a demo 128 bit encryption key,
        chose some another 16 character key for your project.
    */
    unsigned char AESkey[] = "7obdgEn2'TH>]SE|";

    /*
        Demo secret
    */
    unsigned char plainText[] = "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc";

    char errorString[SMARTDNGL_ERROR_STRING_LENGTH_MAX];
    SmartDongleCipher cipher;
    unsigned char *input;
    int plainTextLength;
    int size, error;
    unsigned short address;


    /*
        Calculate the number of AES blocks to be read from SmartDongle,
        rounding block count up to nearest block size.
    */
    plainTextLength = strlen((char *)plainText) + 1; // length + null character
    cipher.AESblockSize = 128;
    cipher.numberOfBlocks =
        (plainTextLength * 8) / cipher.AESblockSize +
        ((plainTextLength * 8 % cipher.AESblockSize > 0) ? 1 : 0);


    /*
        Allocate data string for cipher read from SmartDongle
    */
    size = (cipher.AESblockSize / 8) * cipher.numberOfBlocks;
    input = (unsigned char *)malloc(size);
    if (input  == NULL) {

        error =  SMARTDNGL_ERR_MEM;
        SmartDongleGetErrorString(errorString, error);
        printf("Error %d, %s\n", error, errorString);
        return 1;
    }


    /*
        Read encrypted data back from SmartDongle

        If error code is 41 or 42, the problem may be that the mode of the
        dynamic SmartDongle device files /proc/bus/usb/ and /dev/bus/usb/
        does not allow user access.

        As of linux kernel version 2.6.x, permissions can be set with hal
        facilities.  Refer to HAL/README.txt
    */
    address = 0;
    error =  SmartDongleRead(&P1, &P2, address, input, size);
    if (error) {

        SmartDongleGetErrorString(errorString, error);
        printf("Read Failed %d, %s\n", error, errorString);
        free(input);
        return 1;
    }


    /*
        Decrypt
    */
    cipher.data = input;
    SmartDongleDecode(&cipher, AESkey);


    /*
        Compare to original secret, using orignal secret length.

        The data comparison here is just an exercise, the developer can do
        any number of things with the decrypted user data here.
    */
    if (memcmp(cipher.data, plainText, strlen((char *)plainText)) == 0) {

        printf ("Success: Your encrypted secret was found on SmartDongle\n");
    } else {

        printf ("Fail: Your secret was not found on SmartDongle\n");
    }

    free(input);
    return 0;
}
